//Copyright © 2015, Oracle and/or its affiliates. All rights reserved.

package resource;

import java.net.URI;

import javax.ws.rs.Consumes;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;

import util.StringUtil;
import resource.Department.Departments;
import resource.Employee.Employees;

@Path("departments")
public class DepartmentResource {
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	public Departments listDepartments() {
		Iterable<Department> departments = HRMockDB.INSTANCE.getDepartments();
		return new Departments(departments);
	}

	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	public Response createDepartment(Department department) throws Exception {
		if (department != null) {
			if (department.getDepartmentId() != 0) {
				throw new IllegalArgumentException("Clients should not specify the id");
			}
			return Response.created(doCreateDepartment(department.getDepartmentName())).build();
		}
		throw new IllegalArgumentException("Clients must inform the name of the new department");
	}

	@POST
	public Response createDepartment(@FormParam("name") String name) throws Exception {
		
		return Response.created(doCreateDepartment(name)).build();
	}

	private URI doCreateDepartment(String departmentName) throws Exception {
		if (StringUtil.isNotEmpty(departmentName)) {
			Department createdDepartment = HRMockDB.INSTANCE.createDepartment(departmentName);
			return UriBuilder.fromPath("departments/{id}").build(createdDepartment.getDepartmentId());
		}
		throw new IllegalArgumentException("Clients must inform the name of the new department");
	}

	@GET
	@Path("{id}")
	
	public Department getDepartment(@PathParam("id") int departmentId) {
		return HRMockDB.INSTANCE.getDepartment(departmentId);
	}
	
	@GET
	@Path("{id}/employees")
	@Produces(MediaType.APPLICATION_JSON)
	public Employees listEmployeesByDepartments(@PathParam("id") int departmentId) {
		Iterable<Employee> employees = HRMockDB.INSTANCE.getEmployeesByDepartment(departmentId);
		return new Employees(employees);
	}
	@POST
	@Path("{id}/employees")
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(MediaType.APPLICATION_JSON)
	public Employee createEmployeeInDepartment(@PathParam("id") int departmentId, Employee employee) throws Exception {
		if (employee != null) {
			employee.setDepartmentId(departmentId);
			return HRMockDB.INSTANCE.createEmployee(employee);
		}
		return null;
		
	}
}
